/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.engine.kernel;

import com.sos.scheduler.engine.kernel.util.ClassResource;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;


public final class MavenProperties {
    private static final String resourceSimpleName = "maven.properties";
    private final Properties properties;


    public MavenProperties(ClassResource r) {
        try {
            InputStream in = r.getInputStream();
            try {
                properties = new Properties();
                properties.load(in);
            }
            finally { in.close(); }
        }
        catch (IOException x) { throw new RuntimeException(x); }
    }


    public MavenProperties(Class<?> clas) {
        this(new ClassResource(clas, resourceSimpleName));
    }


    public String getGroupId() {
        return get("project.groupId");
    }
    

    public String getArtifactId() {
        return get("project.artifactId");
    }


    public String getVersion() {
        return get("project.version");
    }


    public String get(String name) {
        String result = (String)properties.get(name);
        if (result == null) throw new RuntimeException("Unknown property '" + name + "'");
        return result;
    }


    @Override public String toString() {
        return getGroupId() + "/" + getArtifactId() + " (" + getVersion() + ")";
    }
}
