/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package net.cwroethel.swt.popupcalendar;

import java.text.SimpleDateFormat;
import java.util.GregorianCalendar;
import java.util.Locale;

/*******************************************************************************
 * Copyright (c) 2005 Will Roethel. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html Contributors: Will Roethel -
 * initial API and implementation
 ******************************************************************************/

/**
 * Utility class providing static methods to get the locale dependent weekday
 * and month names.
 * 
 * @author Will Roethel, http://www.cwroethel.net
 * @version $Revision: 1.1 $
 */
public class CalendarNamesUtil {

    public CalendarNamesUtil() {
    }


    /**
     * Get an array of month names for a given locale.
     * 
     * @param locale Locale
     * @return String[]
     */
    public static String[] getMonthNames(Locale locale) {
        // get a norm calendar using the default locale
        SimpleDateFormat monthFormatter = new SimpleDateFormat("MMMM", locale);
        String[] monthName = new String[12];
        GregorianCalendar cal = new GregorianCalendar(2005, java.util.Calendar.JANUARY, 1);
        for (int iMonth = 0; iMonth < 12; iMonth++) {
            monthName[iMonth] = monthFormatter.format(cal.getTime());
            cal.add(java.util.Calendar.MONTH, 1);
        }
        return monthName;
    }


    /**
     * Get an array with month names for the default locale.
     * 
     * @return String[]
     */
    public static String[] getMonthNames() {
        return getMonthNames(Locale.getDefault());
    }


    /**
     * Get an array of weekday names for a given locale.
     * 
     * @param locale Locale
     * @return String[]
     */
    public static String[] getWeekdayNames(Locale locale) {
        // get a norm calendar using the default locale
        SimpleDateFormat dayFormatter = new SimpleDateFormat("E", locale);
        String[] weekdayName = new String[7];
        GregorianCalendar cal = new GregorianCalendar(2005, java.util.Calendar.JANUARY, 1);
        int currentWeekday = cal.get(java.util.Calendar.DAY_OF_WEEK);
        int daysUntilNextSunday = java.util.Calendar.SATURDAY - currentWeekday + 1;
        cal.add(java.util.Calendar.DAY_OF_MONTH, daysUntilNextSunday);
        for (int iDay = 0; iDay < 7; iDay++) {
            weekdayName[iDay] = dayFormatter.format(cal.getTime()).subSequence(0, 1).toString();
            cal.add(java.util.Calendar.DAY_OF_MONTH, 1);
        }
        return weekdayName;
    }


    /**
     * Get an array of weekday names for the default locale.
     * 
     * @return String[]
     */
    public static String[] getWeekdayNames() {
        return getWeekdayNames(Locale.getDefault());
    }

}
