/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// sostimer.cxx

#include "precomp.h"

#include <limits.h>

#include "../kram/sos.h"
#include "../kram/sosstat.h"
#include "../kram/sostimer.h"

#include <time.h>

// Sos_timer_manager wird von sosstat.cxx initialisiert und ist immer ber
// sos_static_ptr()->_timer_manager erreichbar.

using namespace std;
namespace sos {

DEFINE_SOS_STATIC_PTR( Sos_timer_manager )

//------------------------------------------------------------------------------clock_as_double
// clock_as_double() gives the current time, in seconds, elapsed since 00:00:00 GMT, January 1, 1970.

double clock_as_double()
{
    time_t t;

    return (double)time( &t );
}

//-------------------------------------------------------------------------Sos_timer::Sos_timer

Sos_timer::Sos_timer()
:
    _zero_(this+1)
{
    //obj_const_name( "Sos_timer" );
}

//-------------------------------------------------------------------------Sos_timer::Sos_timer

Sos_timer::Sos_timer( Sos_timer_callback* f, void* callback_data, double time )
:
    _zero_(this+1),
    _callback ( f ),
    _callback_data ( callback_data ),
    _alarm_time ( time )
{
}

//-----------------------------------------------------------------------Sos_timer_node::remove

void Sos_timer_node::remove()
{
    sos_static_ptr()->_timer_manager->remove_timer( this );
}

//-----------------------------------------------------------------Sos_timer_manager::add_timer

Sos_timer_handle Sos_timer_manager::add_timer( Sos_timer_callback* f, void* callback_data, double time )
{
    Sos_timer timer ( f, callback_data, time );

    Sos_ptr<Sos_timer_node>* n = &_list;

    while( *n  &&  (*n)->_timer._alarm_time < time )  n = &(*n)->_tail;

    Sos_ptr<Sos_timer_node> timer_node = SOS_NEW( Sos_timer_node( timer, *n ) );

    *n = timer_node;

    return *n;
}
    
//-----------------------------------------------------------------Sos_timer_manager::add_timer

void Sos_timer_manager::remove_timer( Sos_timer_node* timer_node )
{
    Sos_ptr<Sos_timer_node>* n = &_list;

    while( *n  &&  *n != timer_node )  n = &(*n)->_tail;

    if( +*n != timer_node )  throw_xc( "Sos_timer_manager::remove_timer", "Unbekannter Sos_timer" );

    Sos_ptr<Sos_timer_node> halter = *n;  //? vorsichtshalber

    *n = (*n)->_tail;
}

//-----------------------------------------------------------------Sos_timer_manager::wait_time

double Sos_timer_manager::wait_time()
{
    return _list? max( _list->_timer._alarm_time - clock_as_double(), (double)0.0 )
                : INT32_MAX - 1;
}

//---------------------------------------------------------------------Sos_timer_manager::check

void Sos_timer_manager::check()
{
    while( _list )
    {
        double clock = clock_as_double();
        if( _list->_timer._alarm_time > clock )  break;

        // Timer aus der Liste entfernen
        Sos_ptr<Sos_timer_node> timer_node = _list;
        _list = timer_node->_tail;

        // Callback aufrufen:
        (*timer_node->_timer._callback)( timer_node->_timer._callback_data );
    }
}

//------------------------------------------------------------------------Sos_timer::~Sos_timer

Sos_timer::~Sos_timer()
{
}


} //namespace sos

