/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: spooler_dll_java.cxx 14880 2011-07-22 06:48:54Z jz $

#include "spooler.h"
#include "../zschimmer/java.h"
#include "../kram/sosstat.h"


//#ifdef _DEBUG       // Nur die Debug-Variante wird als DLL erzeugt
//#ifdef Z_WINDOWS

namespace sos 
{
    //int sos_main( int argc, char** argv );
    int process_sos_ini_in_argc_argv(int argc, char** argv);
}


using namespace zschimmer::javabridge;


//-------------------------------------------------------------------------------------------static

static ptr<javabridge::Vm>            static_java_vm;

//---------------------------------------------------------------------------------------JNI_OnLoad

extern "C" JNIEXPORT jint JNICALL JNI_OnLoad( JavaVM* jvm, void* )
{
    CoInitialize(NULL);
    zschimmer_init();
    sos::sos_static_ptr()->add_ref();

    static_java_vm = Z_NEW( javabridge::Vm( jvm ) );
    static_java_vm._ptr->AddRef();                 // Damit bei Programmende nicht Release gerufen wird (die Java-DLL ist dann vielleicht schon entladen)

    return JNI_VERSION_1_4;
}

//-------------------------------------------------------------------------------------JNI_OnUnload

extern "C" JNIEXPORT void JNICALL JNI_OnUnload( JavaVM*, void* )
{
    if( static_java_vm )  static_java_vm._ptr->Release();
    static_java_vm = NULL;

    sos::sos_static_ptr()->remove_ref();
    zschimmer_terminate();
    CoUninitialize();
}

//-----------------------------------------com.sos.scheduler.engine.kernel.main.CppScheduler.runNative

extern "C"
JNIEXPORT int JNICALL Java_com_sos_scheduler_engine_kernel_main_CppScheduler_runNative(
    JNIEnv* jenv, jobject, jobjectArray jargs, jstring argument_line_jstr, jobject java_main_context)
{
    int    result = -1;
    char** argv = NULL;
    int    argc = 0;
    Env    env  = jenv;

    //sos::sos_static_ptr()->init();

    try
    {
        int count = jenv->GetArrayLength( jargs );
        argv = new char*[ count ];

        for( int i = 0; i < count; i++ ) {
            jobject jparam = jenv->GetObjectArrayElement( jargs, i );
            string arg = env.string_from_jstring( (jstring)jparam );
            argv[i] = new char[ arg.length() + 1 ];
            strcpy( argv[i], arg.c_str() );
            argc++;
        }
        argc = sos::process_sos_ini_in_argc_argv(argc, argv);

        result = sos::spooler_main(argc, argv, env.string_from_jstring(argument_line_jstr), java_main_context);
    }
    catch(exception& x) { env.set_java_exception(x); }      

    for( int i = 0; i < argc; i++ )  delete argv[i];
    delete[] argv;

    sos::sos_static_ptr()->close();
    sos::log_stop();

    return result;
}

//-------------------------------------------------------------sos.spooler.Spooler_program.construct
#ifdef Z_WINDOWS

extern "C"
JNIEXPORT void JNICALL Java_sos_spooler_Spooler_1program_construct( JNIEnv* jenv, jobject, jstring parameters_jstr )
{
    try
    {
        int ret = sos::spooler_main( NULL, NULL, Env(jenv).string_from_jstring( parameters_jstr ) );
    }
    catch( const exception&  ) {} //env.set_java_exception(, x ); }
    catch( const _com_error& ) {} //env.set_java_exception(, x ); }
}

//--------------------------------------------------------sos.spooler.Spooler_program.construct_argv

extern "C"
JNIEXPORT void JNICALL Java_sos_spooler_Spooler_1program_construct_1argv( JNIEnv* jenv, jobject, jobjectArray jparams )
{
    char** argv = NULL;
    int    argc = 0;
    Env    env  = jenv;

    try
    {
        int count = jenv->GetArrayLength( jparams );

        argv = new char*[ count ];

        for( int i = 0; i < count; i++ ) {
            jobject jparam = jenv->GetObjectArrayElement( jparams, i );
            string arg = env.string_from_jstring( (jstring)jparam );
            argv[i] = new char[ arg.length() + 1 ];
            strcpy( argv[i], arg.c_str() );
          //fprintf(stderr,"%d: %s\n", i, argv[i]);
            argc++;
        }
        argc = sos::process_sos_ini_in_argc_argv(argc, argv);

        int ret = sos::spooler_main( argc, argv, "" );
    }
    catch( const exception&  ) {} //env.set_java_exception(, x ); }
    catch( const _com_error& ) {} //env.set_java_exception(, x ); }

    for( int i = 0; i < argc; i++ )  delete argv[i];
    delete[] argv;
}

#endif
//-------------------------------------------------------------------------------------------------

//#endif
//#endif
