/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: log_categories.h 13725 2008-11-02 14:55:43Z jz $

#ifndef ZSCHIMMER_LOG_CATEGORIES_H
#define ZSCHIMMER_LOG_CATEGORIES_H

#include "mutex.h"
#include <iostream>


namespace zschimmer {

struct Log_categories;

//------------------------------------------------------------------------------Cached_log_category

struct Cached_log_category
{
                                Cached_log_category         ( const string& name );
                               ~Cached_log_category         ();

    friend ostream&             operator <<                 ( ostream& s, const Cached_log_category& c )  { s << c._name;  return s; }

  private:
    friend struct               Log_categories;

    string                     _name;
    int                        _last_modified_counter;
    bool                       _is_set;
    bool                       _is_valid;
};

//---------------------------------------------------------------------------Log_categories_content

struct Log_categories_content
{
    struct Entry
    {
        enum Type 
        { 
            e_standard,
            e_implicit,         // z.B. "a.b": a => a.b, a schaltet auch a.b an.
            e_explicit,         // z.B. "a.b": a.* schaltet nicht a.b an. a.b muss explizit gesetzt werden. Ebenso bei "all" (="*")
            e_derived           // Abgeleiteter, automatisch generierter Eintrag, zum schnelleren Nachschlagen und fr fehlende Eltern
        };

        static string           type_name                   ( Type );


                                Entry                       ( Type t = e_standard, bool value = false ) : _zero_(this+1), _type(t), _value(value) {}


        Fill_zero              _zero_;
        bool                   _value;
        Type                   _type;

        int64                  _used_counter;
        int64                  _denied_counter;

        bool                   _children_too;               // Benutzer hat gesetzt, z.B. "a.*": alle Kinder, auer e_explicit, setzen
        bool                   _children_too_derived;
        bool                   _has_default;                // Default des Programms 
        bool                   _default_value;
    };

    typedef stdext::hash_map< string, Entry >  Map;


                                Log_categories_content      ()                                      : _zero_(this+1) {}

  protected:

    Fill_zero                  _zero_;
    Map                        _map;
    bool                       _really_all;
};

//-----------------------------------------------------------------------------------Log_categories

struct Log_categories : Log_categories_content
{
    static void                 self_test                   ();


                                Log_categories              ();
                               ~Log_categories              ()                                      {  _valid = false; }

    void                        save_to                     ( Log_categories_content* );
    void                        restore_from                ( const Log_categories_content& );

    void                        set_multiple                ( const string& names );
    void                        set                         ( const string& name );
    string                      set                         ( const string& name, bool value, Entry::Type = Entry::e_standard );
    void                        set_default                 ( const string& name, bool value );         // set_default("misc.java.call")                     => is_set("misc.java.call") == true
    void                        set_local_default           ( const string& name, bool value );         // set_local_default("misc.java.call")  && set("misc.java") => is_set("misc.java.call") == true

    bool                        is_set                      ( const string& name, bool is_derived = false );
    bool                        is_set                      ( const char* name );
    bool                        is_set_cached               ( Cached_log_category* );

    void                    set_all                         ( bool all )                                { _really_all = all;           _modified_counter++; }   // Fr asynchrone Aufrufe (Signal-Handler)
    void                        toggle_all                  ()                                          { _really_all = !_really_all;  _modified_counter++; }   // Fr asynchrone Aufrufe (Signal-Handler)

    int                         modified_counter            () const                                    { return _modified_counter; }
  //bool                        is_modified_since           ( int* last_modified_counter ) const        { bool result = *last_modified_counter != _modified_counter; 
  //                                                                                                      *last_modified_counter = _modified_counter; return result; }
    bool                        is_set                      ( Cached_log_category* c )                  { if( c->_last_modified_counter != _modified_counter )  is_set_cached( c );  
                                                                                                          return c->_is_set; }
    
  //bool                        update_flag_if_modified     ( const char*   name, bool* flag, int* last_modified_counter ) { if( is_modified_since( last_modified_counter ) )  *flag = is_set( name );  return *flag; }
  //bool                        update_flag_if_modified     ( const string& name, bool* flag, int* last_modified_counter ) { if( is_modified_since( last_modified_counter ) )  *flag = is_set( name );  return *flag; }

    bool                        category_is_relevant        ( const string& name ) const;
    Map                         map_copy                    () const;
    string                      to_string                   () const;
    string                      debug_string                () const;

  private:
    friend struct               Log_categories_content;

    bool                        is_set2                     ( const string& name, bool is_derived );
    void                        modify                      ( const Map::iterator&, bool value, bool children_too );
    void                        generate_missing_anchestors_of( const string& );

    Fill_zero                  _zero_;
    bool                       _valid;
    int                        _modified_counter;
    mutable Mutex              _mutex;
};

//-------------------------------------------------------------------------------------------------

} //namespace zschimmer

#endif
